@@ -1,11 +1,15 @@
 import contextlib
-from ._upath import BlobUpath
+from ._blob import BlobUpath
 
 
 class ResourceNotFoundError(Exception):
     pass
 
 
+class ResourceExistsError(Exception):
+    pass
+
+
 class FakeBlobStore:
     '''A in-memory blobstore for illustration purposes'''
 
@@ -15,10 +19,11 @@ class FakeBlobStore:
             'bucket_b': {},
         }
 
-    def write_bytes(self, bucket: str, name: str, data: bytes):
-        print('write bytes', bucket, name, data)
+    def write_bytes(self, bucket: str, name: str, data: bytes, overwrite: bool = False):
+        if name in self._data[bucket] and not overwrite:
+            raise ResourceExistsError
         self._data[bucket][name] = data
-        print('_data:', self._data)
+        return len(data)
 
     def read_bytes(self, bucket: str, name: str):
         z = self._data[bucket]
@@ -58,7 +63,11 @@ class FakeBlobUpath(BlobUpath):
         super().__init__(*parts, bucket=bucket)
         self._bucket = bucket
 
-    def _blob_exists(self):
+    def file_info(self):
+        # place holder
+        return {}
+
+    def isfile(self):
         return _store.exists(self._bucket, self._path)
 
     @contextlib.contextmanager
@@ -72,27 +81,25 @@ class FakeBlobUpath(BlobUpath):
         except ResourceNotFoundError as e:
             raise FileNotFoundError(self) from e
 
-    def _recursive_iterdir(self):
+    def riterdir(self):
         p = self._path
         if not p.endswith('/'):
             p += '/'
-        return _store.list_blobs(self._bucket, p)
+        for pp in _store.list_blobs(self._bucket, p):
+            yield self / pp[len(p):]
 
-    def rm(self, missing_ok=False):
-        if not self.is_file():
+    def rmfile(self, *, missing_ok=False):
+        try:
+            _store.delete_blob(self._bucket, self._path)
+            return 1
+        except ResourceNotFoundError as e:
             if missing_ok:
                 return 0
-            if self.is_dir():
-                raise IsADirectoryError(self)
-            raise FileNotFoundError(self)
-
-        _store.delete_blob(self._bucket, self._path)
-        return 1
-
-    def stat(self):
-        # place holder
-        return {}
+            raise FileNotFoundError(self) from e
 
     def write_bytes(self, data, *, overwrite=False):
-        super().write_bytes(data, overwrite=overwrite)
-        _store.write_bytes(self._bucket, self._path, data)
+        try:
+            _store.write_bytes(self._bucket, self._path,
+                               data, overwrite=overwrite)
+        except ResourceExistsError as e:
+            raise FileExistsError(self) from e
